#!/usr/bin/env node

const program = require('commander')
const path = require('path')
const updateNotifier = require('update-notifier')
const pkg = require('./../package.json')
const shell = require('shelljs')
const notifier = updateNotifier({
  pkg,
  updateCheckInterval: 1000 * 60
})

if (notifier.update) {
  console.log(`有可更新版本: ${notifier.update.latest},建议更新后使用`)
}

// 查看版本
program
  .version(pkg.version, '-v, --version')
// clone项目
// program
//   .command('clone')
//   .description('执行  `sy-cli clone`   初始化项目')
//   .action(option => {
//     let config = {
//       group: null,
//       projects: null
//     }
//     if (!config.group) {
//       let clone = require('./../src/clone')
//       clone(config)
//     }
//   })
// init项目
// program
//   .command('init')
//   .description('执行  `sy-cli init`    创建项目目录及模板文件')
//   .action(option => {
//     let config = {
//       group: null,
//       projects: null
//     }
//     if (!config.group) {
//       let init = require('./../src/init')
//       init(config)
//     }
//   })
// upload上传文件到服务器
program
  .command('upload <path>')
  .option('--am', '监听是否为提交')
  .option('-t,--test', '监听是否为test分支')
  .description('执行  `sy-cli upload --am` 前端统一部署工具')
  .action((path, cmd) => {
    let config = {
      path: path,
      am: cmd.am,
      test: cmd.test
    }
    let upload = require('./../src/upload')
    upload.init(config)
  })
program
  .command('utils')
  .description('执行  `sy-cli util`    提供基础公共方法')
  .action(option => {
    let utils = require('./../src/utils')
    utils()
  })
/* program
  .command('iconfont')
  .description('执行  `sy-cli iconfont`    提供iconfont方法')
  .action(option => {
    let utils = require('./../src/icon')
    utils()
  }) */
program
  .command('bsync')
  .description('执行  `sy-cli bsync`     开启后文件保存,浏览器同步刷新')
  .action(option => {
    let config = {
      address: null,
      dist: null
    }
    if (!config.address) {
      let bsync = require('./../src/bsync')
      bsync(config)
    }
  })

// program
//   .command('cdn')
//   .description('执行  `sy-cli cdn`    cdn相关操作')
//   .action(option => {
//     let cdn = require('./../src/cdn')
//     cdn()
//   })
// 调用当前目录package.json里面 script的命令
try {
  let pwdPkg = require(`${shell.pwd()}/package.json`)
  if (pwdPkg.scripts) {
    let obj = pwdPkg.scripts
    let arr = []
    let txt = '执行 '
    Object.keys(obj).forEach(el => {
      arr.push(el)
    })
    txt += ` \`sy-cli r ${arr.join('|')}\` 当前目录的方法`
    program
      .command(`r <option>`)
      .description(txt)
      .action(option => {
        let scripts = require('./../src/scripts')
        scripts(option)
      })
  }
} catch (error) {

}
program
  .command('sprite [path]')
  .description('执行  `sy-cli sprite`    提供生成精灵图及映射css方法')
  .action(jsonPath => {
    let config
    if (jsonPath) {
      config = require(path.resolve(shell.pwd().toString(), jsonPath))
    } else {
      config = {
        address: null,
        prefix: null
      }
    }
    let utils = require('./../src/sprite')
    utils(config)
  })

program
  .command('qr <path>')
  .description('执行  `sy-cli qr <path>`  自动生成二维码')
  .action(path => {
    let utils = require('./../src/qr')
    utils.init(path)
  })

program
  .command('cdn <path>')
  .description('执行  `sy-cli cdn <path>`  上传七牛cdn')
  .action(confPath => {
    let utils = require('./../src/cdn')
    utils(confPath)
  })

program.parse(process.argv)
