const shell = require('shelljs')
const path = require('path')
let config
let utils
const QR = {
  picLogin: 'QR/login.png',
  picPreview: 'QR/preview.png',
  login () {
    let _this = this
    shell.rm('-rf', 'QR')
    shell.mkdir('QR')
    shell.exec(`${utils.cli} -l --login-qr-output image@${path.resolve(process.cwd(), _this.picLogin)}`, (code, stdout, stderr) => {
      if (stdout.indexOf('login success') > -1) {
        _this.createQR()
      } else if (+code !== 0) {
        console.log('登录失败')
        process.exit(1)
      }
    })
    setTimeout(() => {
      let _txt = `小程序编译,请扫码登录,${_this.toDate(new Date())}`
      shell.exec(_this.fmtJson(_txt, _this.picLogin, '扫码登录'))
    }, 2000)
  },
  fmtJson (txt, url, title) {
    let _time = Date.parse(new Date())

    return [
      `curl 'https://oapi.dingtalk.com/robot/send?access_token=${utils.dingtalkToken}'`,
      `-H 'Content-Type: application/json'`,
      `-d '`,
      `{"msgtype": "actionCard",`,
      `"actionCard": {`,
      `"text": "${txt}",`,
      `"title": "${txt}",`,
      `"hideAvatar": "0",`,
      `"btnOrientation": "1",`,
      `"btns": [`,
      `{`,
      `"title": "${title}",`,
      `"actionURL": "${utils.jenkinsURL}${url}?t=${_time}"`,
      `}`,
      `]`,
      `}`,
      `}'`
    ].join(' ')
  },
  createQR () {
    let _this = this
    shell.rm('-rf', 'QR')
    shell.mkdir('QR')
    shell.exec(`${utils.cli} -p ${utils.distProject} --preview-qr-output image@${path.resolve(process.cwd(), _this.picPreview)}`, (code, stdout, stderr) => {
      if (stderr.indexOf('需要重新登录') > -1) {
        _this.login()
      } else if (+code !== 0) {
        console.log('生成失败')
        process.exit(1)
      } else {
        let _txt = `小程序编译成功,请扫码查看,${_this.toDate(new Date())}`
        shell.exec(_this.fmtJson(_txt, _this.picPreview, '扫码查看'))
      }
    })
  },
  build () {
    return new Promise((resolve, reject) => {
      shell.cd(`${utils.sourceProject}`)
      shell.exec(`${utils.sourceScripts}`, (code, stdout, stderr) => {
        if (code) {
          reject(stderr)
        } else {
          resolve(1)
        }
      })
    })
  },
  async init (pathName) {
    let _this = this
    let _jsonPath = path.join(process.cwd(), pathName)
    try {
      config = require(_jsonPath)
      this.initData()
      let _build = await _this.build().catch((err) => {
        console.log(err)
        console.log('编译失败')
        process.exit(1)
      })
      if (_build) {
        _this.createQR()
      }
    } catch (error) {
      console.error(error)
    }
  },
  initData () {
    utils = {
      sourceProject: path.join(process.cwd()),
      cli: config.path.cli,
      sourceScripts: config.source.scripts,
      dingtalkToken: config.dingtalkToken,
      jenkinsURL: config.jenkinsURL,
      distProject: path.resolve(process.cwd(), config.path.wx)
    }
  },
  toDate (timeStamp, getMilliseconds) {
    let date = new Date(timeStamp)
    let time = [
      date.getFullYear(),
      date.getMonth() + 1,
      date.getDate(),
      date.getHours(),
      date.getMinutes(),
      date.getSeconds()
    ]

    getMilliseconds && time.push(date.getMilliseconds())

    for (var i = 0; i < time.length; i++) {
      var val = time[i]
      if (val <= 9) {
        time[i] = '0' + val
      }
    }
    if (getMilliseconds) {
      time = time.slice(0, 3).join('-') + ' ' + time.slice(3, time.length).join(':')
    } else {
      time = time.slice(0, 3).join('-') + ' ' + time.slice(3, 6).join(':')
    }
    return time
  }
}

module.exports = QR
