/**
 * 留给未来上传测试服务器使用
 */
const shell = require('shelljs')
const path = require('path')
const pkg = require('../package.json')
let _root = 'publish-view'
let parBranch = ''
let subBranch = ''
let config = null
const Git = {
  LogMessage () {
    [
      `由于出现了不满足联调等多种环境的情况，请先更新sy-cli版本,最新版本为${pkg.version},以免出现意外导致无法编译的情况`,
      '新版本将不再需要 (--test | -t) 参数 ，并且一个分支一个json文件',
      `当前项目分支为${parBranch},执行命令为 sy-cli upload <path> --test.`,
      '具体使用说明请查看wiki中构建步骤,若有疑问，请及时联系。'
    ].forEach(v => console.log(v))
  },
  jsonFileParamMsg () {
    [
      '当前执行命令  sy-cli up  load <path> --test 但是配置项（.json文件中）未配置 "subTestBranch",请配置！',
      '若您的配置文件中有了"subBranch"，并将sy-cli更新到了最新版本，那么请将您参数中的(--test | -t )去掉',
      '按照wiki中部署流程，配置对应json文件，若有疑问，请及时联系。'
    ].forEach(v => console.log(v))
  },
  subBranchMsg () {
    [
      '请将您的sy-cli 升级到最新版本, 并且将json文件中的subBranch配置,例如子项目为master ,那么请配置  ',
      '"subBranch":"master"',
      '具体信息，请前往wiki中查看对应配置，若有疑问，请及时联系。'
    ].forEach(v => console.log(v))
  },

  checkout (test) {
    let _url = (config.protocol || 'http://') + config.username + ':' + config.password + '@' + config.url
    let _cmd = 'git submodule add'
    let distRoot = process.platform === 'win32' ? _root : path.join(process.cwd(), _root)
    if (test) {
      this.LogMessage()
      if (!config.subTestBranch) {
        this.jsonFileParamMsg()
        process.exit(1)
      }
      _cmd = `git submodule add -b ${config.subTestBranch}`
    } else if (config.subBranch) {
      _cmd = `git submodule add -b ${config.subBranch}`
    } else {
      this.subBranchMsg()
    }
    shell.exec(`git rm -r --cached ${_root}`)
    shell.exec(`${_cmd} ${_url} ${distRoot}`, (code, stdout, stderr) => {
      console.log(code)
      if (code) {
        console.log(stderr)
        process.exit(1)
      }
    })
  },
  async pull () {
    let rootPath = path.join(process.cwd(), _root)
    shell.cd(rootPath)
    let subBranch = await this.getBranch()
    let branch = config.subBranch ? config.subBranch : config.subTestBranch
    if (subBranch !== branch) {
      console.log('分支错误！请按照wiki中部署流程，配置对应json文件，若有疑问，请及时联系。')
      process.exit(1)
    }
    return new Promise((resolve, reject) => {
      shell.exec('git remote -v')
      shell.exec(`git pull`, (code, stdout, stderr) => {
        if (code) {
          console.log(stderr)
          process.exit(1)
        }
        resolve()
      })
    })
  },
  async commit () {
    let _gitLog = await this.getLog()
    await this.pull()
    let _msg = `${config.productName} - commit:${_gitLog.commit.substr(0, 8)} - ${_gitLog.message}  - author:${_gitLog.author} - date:${_gitLog.date}`
    subBranch = await this.getBranch()
    console.log(`当前项目分支为${parBranch},子项目分支为${subBranch}`)
    shell.exec(`git add .`)
    shell.exec(`git commit -am '${_msg}'`)
    shell.exec(`git push`)
    this.getCommitFiles().then(arr => {
      console.log(arr)
    })
  },
  getCommitFilesTxt: 'git log 获取文件修改记录',
  getCommitFiles () {
    let _this = this
    return new Promise((resolve) => {
      // git log 以json形式返回
      let _txt = `\n ---${_this.getCommitFilesTxt}---`
      let gitlog = `git log -m  -1 `
      // let gitlog = `git log origin/master  -1 `
      let returnArr = []
      shell.exec(gitlog)
      gitlog += ` --name-status --pretty=format:'${_txt}'`
      shell.exec(gitlog, (code, stdout, stderr) => {
        let str = stdout.replace(new RegExp(_txt, 'g'), '')
        str = str.split('\n')
        returnArr = str.filter((v) => {
          if (v) return v
        })
        returnArr = returnArr.map((v, i) => {
          let s = v.split('\t')
          return {
            'status': s[0],
            'filename': s[1]
          }
        })
        resolve(returnArr)
      })
    })
  },
  getLog () {
    let _cmd = `git log -1 \
  --date=iso --pretty=format:'{"commit": "%H","author": "%aN <%aE>","date": "%ad","message": "%s"},' \
  $@ | \
  perl -pe 'BEGIN{print "["}; END{print "]\n"}' | \
  perl -pe 's/},]/}]/'`
    return new Promise((resolve, reject) => {
      shell.exec(_cmd, (code, stdout, stderr) => {
        if (code) {
          reject(stderr)
        } else {
          try {
            resolve(JSON.parse(stdout)[0])
          } catch (error) {
            console.log('获取git log 失败，请检查msg中是否有空格或者特殊字符')
            process.exit(1)
          }
        }
      })
    })
  },
  getBranch () {
    return new Promise((resolve, reject) => {
      shell.exec('git branch', (code, stdout, stderr) => {
        if (code) {
          reject(stderr)
        } else {
          resolve(stdout.split('\n').find(n => n.indexOf('*') > -1).replace(/\*\s/g, ''))
        }
      })
    })
  },
  async init (options) {
    let _jsonPath = path.join(process.cwd(), options.path)
    try {
      config = require(_jsonPath)

      if (config.cwdpath && typeof config.cwdpath === 'string') {
        shell.cd(config.cwdpath)
      }

      if (config.subProduct && typeof config.subProduct === 'string') {
        _root = config.subProduct
      } else {
        console.log(`请将自己项目中 ${options.path} 配置文件增加"subProduct"字段。 \r 例如 "subProduct": "publish-view".`)
      }
      // 当前项目分支
      parBranch = await this.getBranch()
      if (options.am) {
        this.commit()
      } else {
        if (shell.find(_root).code) {
          this.checkout(options.test)
        } else {
          this.pull()
        }
      }
    } catch (error) {
      console.error(error)
    }
  }
}
module.exports = Git
