const clc = require('cli-color')
const fs = require('fs')
const Spritesmith = require('spritesmith')
const path = require('path')
const templater = require('spritesheet-templates')

function readFileList (path, filesList, prefix) {
  let files = fs.readdirSync(path)
  files.forEach((itm, index) => {
    let stat = fs.statSync(path + itm)
    if (stat.isDirectory()) {
      // 递归读取文件
      readFileList(path + itm + '/', filesList, prefix)
    } else {
      if (new RegExp('^' + prefix + '.*(gif|jpg|jpeg|png|GIF|JPG|PNG)$', 'ig').test(itm)) {
        filesList.push({
          path, // 路径
          itm // 名字
        })
      }
    }
  })
}

function getCoordinateName (filepath, cb) {
  let nameParts = path.basename(filepath).split('.')
  if (nameParts.length >= 2) {
    nameParts.pop()
  }
  return nameParts.join('.')
}
module.exports = function (params, flag) {
  let sourcePath = params.address
  let filesList = params.filesList || []

  if (!flag) readFileList(sourcePath + '/', filesList, params.prefix)

  if (filesList.length) {
    Spritesmith.run({
      src: filesList.map(v => path.join(v.path, v.itm)),
      padding: params.padding
    }, function handleImages (err, result) {
      if (err) return console.error(err)
      let spritePath = path.resolve(sourcePath, 'sprite')
      fs.mkdir(spritePath, () => {
        let spritePng = path.resolve(spritePath, `${params.prefix}-sprite.png`)
        let spriteCss = path.resolve(spritePath, `${params.prefix}-sprite.css`)
        fs.writeFileSync(spritePng, result.image)
        let properties = result.properties
        let cssPar = {
          spritesheet: {
            width: properties.width,
            height: properties.height,
            image: `./${params.prefix}-sprite.png`
          }
        }
        cssPar.sprites = Object.getOwnPropertyNames(result.coordinates).sort().map((file) => {
          let name = getCoordinateName(file)
          let coords = result.coordinates[file]
          coords.name = name
          coords.source_image = file
          coords.image = spritePath
          coords.total_width = properties.width
          coords.total_height = properties.height
          return coords
        })
        fs.writeFile(spriteCss, templater(cssPar, {
          format: 'css'
        }).replace(/("([^\\"]*(\\.)?)*")|('([^\\']*(\\.)?)*')|(\/{2,}.*?(\r|\n))|(\/\*(\n|.)*?\*\/)/g, ''), 'utf8', function (error) {
          if (error) {
            return console.log(error)
          }

          console.log(clc.yellow.bold('创建成功，目录为：' + spritePath))
          console.log(clc.yellow.bold('图片： ' + spritePng))
          console.log(clc.yellow.bold('样式表: ' + spriteCss))
        })
      })
    })
  } else {
    console.log(clc.yellow.bold(`您提供的目录下不存在以 -- ${params.prefix} -- 开头的图片`))
    console.log(clc.yellow.bold(`请检查……`))
  }
}
